#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-11-27
# Last Modified: $Date: 2008-12-01 14:54:22 +0000 (Mon, 01 Dec 2008) $
# Id:            $Id: archive_from_tarball_main.pm 3652 2008-12-01 14:54:22Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/npg-qc/tags/release-6.0/lib/npg_qc/api/archive_from_tarball_main.pm $
#

package npg_qc::api::archive_from_tarball_main;
use strict;
use warnings;
use Class::Std;
use Carp;
use English qw{-no_match_vars};
use base qw(npg_qc::api::base);
use Perl6::Slurp;
use npg_qc::api::process::bustard_output;
use srpipe::util;
use Readonly;
use npg_qc::api::run_caching;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3652 $ =~ /(\d+)/mx; $r; };

Readonly our $TRIES => 10;
Readonly our $TRY_SUCCESS => -1;
Readonly our $TEMP_ID_RUN => 5;

Readonly our %NUM_TILE_PER_RUN => (
                                   2640 => 1,
                                   800  => 1,
                                   1600 => 1,
                                   2400 => 1,
                                  );

{
  ## no critic
  my %srpipe_util_of        :ATTR( :get<srpipe_util>, :set<srpipe_util>);
  my %bustard_file_urls_of  :ATTR( :get<bustard_file_urls>, :set<bustard_file_urls> );
  my %bustard_xml_output_of :ATTR( :get<bustard_output>, :set<bustard_output> );
  my %response_objects_of   :ATTR( :get<response_objects>, :set<response_objects> );
  my %file_root_of          :ATTR( :get<file_root>, :set<file_root> );
  my %file_dir_of           :ATTR( :get<file_dir>, :set<file_dir> );
  my %file_ext_of           :ATTR( :get<file_ext>, :set<file_ext> );
  my %summary_dir_of        :ATTR( :get<summary_dir>, :set<summary_dir> );
  my %fastqcheck_dir_of     :ATTR( :get<fastqcheck_dir>, :set<fastqcheck_dir> );
  my %bz_id_run             :ATTR( :get<bz_id_run>, :set<bz_id_run> );
  my %bz_id_run_pair        :ATTR( :get<bz_id_run_pair>, :set<bz_id_run_pair> );
  my %run_cache_of          :ATTR( :get<run_cache_object>, :set<run_cache_object> );
  ## use critic

  #for processing bustard files
  sub process_all_bustard {
    my ($self) = @_;

    my $files_to_process = $self->retrieve_bustard_file_urls();

    if (!$files_to_process) {
      croak 'No files to process at this time!';
    }

    return;
  }

  sub retrieve_bustard_file_urls {
    my ($self) = @_;

    my $util = $self->util();

    my $has_files_to_process = 0;

    my $id_run = $self->get_bz_id_run();

    carp "Processing run : $id_run\n";
    $self->set_bustard_output([]);
    $self->set_bustard_file_urls([]);
    my @files;
    $has_files_to_process++;
    my $srpipe_util = srpipe::util->new();
    my $file_dir    = $self->get_file_dir();
    $file_dir       =~ s/run_no/$id_run/gxms;
    $file_dir       .= $self->get_file_ext();
    @files          = $self->find_file_urls($file_dir);

    my $prelim_analysis;

    foreach my $file (@files) {
      if ($file =~ /config/xms) {
        my $text = slurp($file);
        if ($text =~ /This\ is\ a\ every-20th-tile\ run/xms) {
          $prelim_analysis++;
        }
        last;
      }
    }

    if ($prelim_analysis) {
      $has_files_to_process--;
      carp "nothing for $id_run";
      next;
    }
    my $summary_dir = $self->get_summary_dir();
    $summary_dir    =~ s/run_no/$id_run/gxms;
    $self->find_file_urls($summary_dir);

    my$fastqcheck_dir = $self->get_fastqcheck_dir();
    $fastqcheck_dir   =~ s/run_no/$id_run/gxms;
    $self->find_file_urls($fastqcheck_dir);

    undef $srpipe_util;

    if (!$self->get_bustard_file_urls()->[0]) {
      $has_files_to_process--;
      carp "nothing for $id_run";
      next;
    }

    my ($file_for_run) = $self->get_bustard_file_urls()->[0] =~ /IL\d+_0*(\d+)/xms;
    if ($file_for_run != $id_run) {
      $has_files_to_process--;
      carp "nothing for $id_run";
      next;
    }

    $self->retrieve_bustard_xml();
    if (!$self->get_bustard_output() || scalar@{$self->get_bustard_output()} == 0) {
      $has_files_to_process--;
      carp "nothing for $id_run";
      next;
    }

    $self->collate_qcal_xml();
    $self->websend_bustard_xml();
    $self->process_response_objects();
    my $id_run_pair = $self->get_bz_id_run_pair() || $id_run;
    $self->run_caching({id_run => $id_run, id_run_pair => $id_run_pair});

    return $has_files_to_process;
  }

  sub run_caching{
    my ($self, $arg_refs) = @_;

    my $id_run = $arg_refs->{id_run};
    my $not_double_ended;

    if ($id_run == $arg_refs->{id_run_pair}) {
      $not_double_ended++;
    }

    my $util = $self->util();
    my $api_run_caching;

    if ($self->get_run_cache_object) {
      $api_run_caching = $self->get_run_cache_object();
    } else {
      $api_run_caching = npg_qc::api::run_caching->new({util => $util});
    }

    print " caching lane summary data and run graph\n" or croak "can't print\n";

    eval{
      $api_run_caching->cache_query_lane_summary($id_run, 1, 1);
      $api_run_caching->run_graph($id_run, 1, 1);
      if (!$not_double_ended) {
        $api_run_caching->cache_query_lane_summary($id_run, 2, 1);
        $api_run_caching->run_graph($id_run, 2, 1);
      }
      1;
    } or do {
      croak $EVAL_ERROR;
    };

    return;
  }

  sub find_file_urls {
    my ($self, $file_dir) = @_;

    my $srpipe_util;
    if (!($srpipe_util = $self->get_srpipe_util())) {
      $srpipe_util = srpipe::util->new();
      $self->set_srpipe_util($srpipe_util);
    }

    my @files = $srpipe_util->my_glob($self->get_file_root(), $file_dir);

    if ($files[0] && $files[0] !~ /tmp/xms) {
      push @{$self->get_bustard_file_urls()}, @files;
    }

    return @files;
  }

  sub collate_qcal_xml {
    my ($self) = @_;
    my $qcals = [];
    my $parser = $self->util->parser();
    my $xmls;

    foreach my $xml (@{$self->get_bustard_output()}) {
      if ($xml !~ /<dataset>qcal<\/dataset/xms) {
        push @{$xmls}, $xml;
        next;
      }
      push @{$qcals}, $xml;
    }

    if (scalar@{$qcals} == 0) {
      return 1;
    }

    my @rows;
    foreach my $xml (@{$qcals}) {
      my $parsed_xml = $parser->parse_string($xml);
      my $row = $parsed_xml->getElementsByTagName('row')->[0];
      push @rows, $row;
    }

    my $xml = qq{<?xml version="1.0" encoding="utf-8"?><root>\n<dataset>qcal</dataset>\n<rows>\n};
    foreach my $row (@rows) {

      #############
      # ensures no null values sent
      my $twenty = $row->getAttribute('twenty');
      $twenty ||= 0;
      $row->setAttribute('twenty', $twenty);
      my $twentyfive = $row->getAttribute('twentyfive');
      $twentyfive ||= 0;
      $row->setAttribute('twentyfive', $twentyfive);
      my $thirty = $row->getAttribute('thirty');
      $thirty ||= 0;
      $row->setAttribute('thirty', $thirty);

      $xml .= $row->toString();
    }
    $xml .= qq{\n</rows>\n</root>};

    push @{$xmls}, $xml;
    $self->set_bustard_output($xmls);

    return 1;
  }

  sub retrieve_bustard_xml {
    my ($self) = @_;
    my $util = $self->util();

    foreach my $file_url (@{$self->get_bustard_file_urls()}) {

      my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});

      if (!$self->get_bustard_output()) {
        $self->set_bustard_output([]);
      }
      my $xml = $bustard_output->convert_to_xml($file_url);
      if ($xml) {
        if ($xml =~ m/\<dataset\>summary_data\<\/dataset\>/xms) {
          push @{$self->get_bustard_output()}, $bustard_output->reprocess_summary($xml);
        } else {
          push @{$self->get_bustard_output()}, $xml;
        }
      }

    }

    return;
  }

  sub websend_bustard_xml {
    my ($self) = @_;
    my $util = $self->util();

    my $return_value;

    carp 'Starting to load.';

    foreach my $xml (@{$self->get_bustard_output()}) {
      my ($dataset) = $xml =~ m/\<dataset\>(.*?)\<\/dataset\>/xms;
      carp $dataset;
      $return_value = $self->websend_any_xml($xml);
    }

    ##########
    # return the last value returned by websend_any_xml, in case it is useful
    return $return_value;
  }

  sub process_response_objects {
    my ($self) = @_;

    my $parser = $self->util->parser();
    my $sent_type_counts = {};

    foreach my $response (@{$self->get_response_objects()}) {
#    warn $response->content;
     my $sent_type;
      eval {
        $sent_type = $parser->parse_string($response->content)->getElementsByTagName('sent_type');
        $sent_type_counts->{$sent_type}++;
        1;
      } or do {
        carp $response->content();
        carp 'error in xml response object';
      };
    }

    $self->set_response_objects(undef);

    return $sent_type_counts;
  }

  sub websend_any_xml {
    my ($self, $xml) = @_;
    my $util = $self->util();

    if ($xml !~ m/\<dataset\>run_config\<\/dataset\>/xms) {
      my $parsed_xml = $util->parser->parse_string($xml);
    }

    if (!$self->get_response_objects()) {
      $self->set_response_objects([]);
    }

    my $response = $util->post(
      $util->base_uri().q{create_xml},
      [
        'Content_Type'   => 'application/xml',
                        'Content_Length' => length $xml,
                        'Content'        => $xml,
                        'Accept'         => 'text/xml',
                        ]
    );

    if ($response->is_success()) {
      push @{$self->get_response_objects()}, $response;
    } else {
      carp 'failed to successfully post - this could be from a timeout issue';
      return 0;
    }

    return 1;
  }

}
1;
__END__

=head1 NAME

npg_qc::api::archive_from_tarball_main

=head1 VERSION

$LastChangedRevision: 3652 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 new - Constructor

  my $oDerived = npg_qc::api::main->new({'util' => $oUtil,});
  
    my $oDerived = npg_qc::api::main->new({
    # 'key' => value parameters for the derived class
  });

=head2  find_file_urls -
=head2  run_caching -

=head2 process_all_bustard - handler to run and it does all in correct order for you, with no need to go through each method in turn in script

=head2 collate_qcal_xml - handler collates all the qcal information into one xml for sending to the web, as each only has one row ordinarily

=head2 retrieve_bustard_file_urls - finds all the relevant Bustard file uris and stores them 

=head2 retrieve_bustard_xml - retrieves bustard files, and processes them via process::bustard_output into xml

=head2 websend_bustard_xml - sends all the bustard xml strings to npg-qc via LWP::UserAgent

=head2 websend_any_xml - sends a single xml string to npg-qc via LWP::UserAgent

=head2 process_response_objects - processes the response objects from websend_xml

=head2 util - util instance for this resource set

  Returns a cached object if one exists, or creates and caches a new one

    my $oUtil = $oDerived->util();
    
=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
Carp
English
npg_qc::api::base
Perl6::Slurp
npg_qc::api::process::bustard_output
srpipe::util
Readonly
npg_qc::api::run_caching

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS
=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown
This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
