#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-11-21
# Last Modified: $Date: 2008-12-01 14:54:22 +0000 (Mon, 01 Dec 2008) $
# Id:            $Id: loader.pm 3652 2008-12-01 14:54:22Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/npg-qc/tags/release-6.0/lib/npg_qc/api/image/loader.pm $
#
package npg_qc::api::image::loader;
use base qw(Class::Accessor npg_qc::api::base);
use strict;
use warnings;
use Carp;
use Class::Std;
use npg_qc::api::util;
use English qw(-no_match_vars);
use Readonly;
use Perl6::Slurp;

our $VERSION     = do { my ($r) = q$LastChangedRevision: 3652 $ =~ /(\d+)/mxs; $r; };
our $DEBUG       = 0;
Readonly our $MAX_RETRIES => 10;
Readonly our $RETRY_DELAY => 10;

{
  ## no critic
  my %image_names_of :ATTR( :get<image_names>, :set<image_names> );
  my %image_dir_root_of :ATTR( :get<dir_root>, :set<dir_root> );
  my %current_name_of :ATTR( :get<current_image_name>, :set<current_image_name> );
  my %thumbnail_of :ATTR( :get<thumbnail>, :set<thumbnail> );
  my %id_run_of :ATTR( :get<id_run>, :set<id_run> );
  my %type_of :ATTR( :get<type>, :set<type> );
  my %suffix_of :ATTR( :get<suffix>, :set<suffix> );
  my %image_of :ATTR( :get<image>, :set<image> );
  ## use critic
  
  sub image_loop_from_file {
    my ($self) = @_;

    my $dbh = $self->util->get_dbh();

    eval {
      foreach my $image_name (@{$self->get_image_names()}) {
        my $image_url = $self->get_dir_root().q{/}.$image_name;
        my $image = slurp($image_url);
        $self->set_current_image_name($image_name);
        $self->set_image($image);
        $self->load_into_database;
      }
      1;
    } or do {
      $dbh->rollback();
      croak $EVAL_ERROR;
    };

    $dbh->commit();

    return 1;
  }
  
  sub load_into_database {
    my ($self) = @_;

    my $image_name = $self->get_current_image_name();

    my ($suffix) = $image_name =~ /\.(.*?)\z/xms;
    $suffix = $self->get_suffix() || $suffix;
    $suffix ||= 'png';

    my $query = q{INSERT INTO image_store(type, thumbnail, image, image_name, id_run, suffix) VALUES (?,?,?,?,?,?)};

    my $dbh = $self->util->get_dbh();
    eval {
      $dbh->do($query, {},
        $self->get_type(),
        $self->get_thumbnail(),
        $self->get_image(),
        $self->get_current_image_name(),
        $self->get_id_run(),
        $suffix
        );
    } or do {
      croak $EVAL_ERROR;
    };
    $dbh->commit();
    return 1;
  }
}
1;
__END__
=head1 NAME

npg_qc::api::image::loader

=head1 VERSION

$LastChangedRevision: 3652 $

=head1 SYNOPSIS

  my $oImageLoader = npg_qc::api::image::loader->new({util => $util});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 set_suffix - if image_names do not have a suffix, and are not png images, then use this to store a suffix in the database

  $oImageLoader->set_suffix($suffix);

=head2 image_loop_from_file - goes through a list of file_names with a dir root and loads them into the database

  $oImageLoader->set_image_names([qw(im01.png im02.png im03.png)]);
  $oImageLoader->set_dir_root('dir/root/to/images');
  eval { $oImageLoader->image_loop_from_file(); } or do { croak $EVAL_ERROR; };

=head2 load_into_database - loads an image into the database.

  $oImageLoader->set_type($type);
  $oImageLoader->set_thumbnail($boolean);
  $oImageLoader->set_image($image);
  $oImageLoader->set_current_image_name($image_name);
  $oImageLoader->set_id_run($id_run)
  eval { $oImageLoader->load_into_database(); } or do { croak $EVAL_ERROR; };

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

Class::Accessor
npg_qc::api::base
Carp
Class::Std
npg_qc::api::util
English
Readonly
Perl6::Slurp

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>, & Roger Pettett, E<lt>rmp@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
