#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-11-14
# Last Modified: $Date: 2008-12-01 14:54:22 +0000 (Mon, 01 Dec 2008) $
# Id:            $Id: signal_mean.pm 3652 2008-12-01 14:54:22Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/npg-qc/tags/release-6.0/lib/npg_qc/api/signal_mean.pm $
#
package npg_qc::api::signal_mean;
use strict;
use warnings;
use Class::Std;
use Carp;
use base qw(npg_qc::api::base);
use Readonly;
use JSON;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3652 $ =~ /(\d+)/mx; $r; };

{
  ## no critic
  my %id_run_of         :ATTR( :get<id_run>,          :set<id_run>          );
  my %position_of       :ATTR( :get<position>,        :set<position>        );
  my %all_a_of          :ATTR( :get<all_a>,           :set<all_a>           );
  my %all_c_of          :ATTR( :get<all_c>,           :set<all_c>           );
  my %all_g_of          :ATTR( :get<all_g>,           :set<all_g>           );
  my %all_t_of          :ATTR( :get<all_t>,           :set<all_t>           );
  my %base_a_of         :ATTR( :get<base_a>,          :set<base_a>          );
  my %base_c_of         :ATTR( :get<base_c>,          :set<base_c>          );
  my %base_g_of         :ATTR( :get<base_g>,          :set<base_g>          );
  my %base_t_of         :ATTR( :get<base_t>,          :set<base_t>          );
  my %call_a_of         :ATTR( :get<call_a>,          :set<call_a>          );
  my %call_c_of         :ATTR( :get<call_c>,          :set<call_c>          );
  my %call_g_of         :ATTR( :get<call_g>,          :set<call_g>          );
  my %call_t_of         :ATTR( :get<call_t>,          :set<call_t>          );
  my %cycle_of          :ATTR( :get<cycle>,           :set<cycle>           );
  my %signal_means_of   :ATTR( :get<signal_means>,    :set<signal_means>    );
  ## use critic

  sub fields {
    my ($self) = @_;
    return qw{
      id_run
      position
      all_a
      all_c
      all_g
      all_t
      base_a
      base_c
      base_g
      base_t
      call_a
      call_c
      call_g
      call_t
      cycle
    };
  }

  sub id_run {
    my ($self, $id_run) = @_;
    if ($id_run) {
      $self->set_id_run($id_run);
    }
    return $self->get_id_run();
  }

  sub read_signal_mean_data {
    my ($self) = @_;
    my $util = $self->util();
    my $pkg        = ref $self;
    my ($obj_type) = $pkg =~ /([^:]+)$/mx;
    my $obj_uri    = sprintf '%s/%s.json?id_run=%s', $util->base_uri(), $obj_type, $self->id_run();
    my $response   = $util->get($obj_uri);

    if(!$response->is_success()) {
      croak $response->status_line();
    }

    my $response_hash = from_json($response->content());
    my $id_run = $response_hash->{id_run};
    $self->id_run($id_run);

    foreach my $sm (@{$response_hash->{signal_means}}) {
      $sm->{id_run} = $id_run;
      $sm = $self->new_from_json($pkg, $sm);
    }

    $self->set_signal_means($response_hash->{signal_means});

    return 1;
  }

  sub signal_means_by_position {
    my ($self, $position) = @_;

    $position ||= $self->get_position();

    if (!$self->get_signal_means()) {
      $self->read_signal_mean_data();
    }

    my $wanted_signal_means;
    foreach my $sm (@{$self->get_signal_means()}) {
      if ($sm->get_position() == $position) {
        push @{$wanted_signal_means}, $sm;
      }
    }

    return $wanted_signal_means;
  }

}
1;
__END__

=head1 NAME

npg_qc::api::signal_mean

=head1 VERSION

$LastChangedRevision: 3652 $

=head1 SYNOPSIS

  my $oSignalMean = npg_qc::api::signal_mean->new({util => $oUtil});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 fields - returns an array of fieldnames for this object

  my $aFields = $oSignalMean->fields();

=head2 id_run - accessor for id_run. takes an id as an optional parameter, which it stores, and then returns the id_run

  my $IdRun = $oSignalMean->id_run($id_run);

=head2 read_signal_mean_data - requests the signal mean data for the id_run stored, and stores this as dignal mean objects in memory

  eval { $oSignalMean->read_signal_mean_data(); } or do { croak $EVAL_ERROR; };
  my $aSignalMeans = $oSignalMean->get_signal_means();

=head2 signal_means_by_position - returns an array of signal mean data objects jsut for the given lane. This can either have been set, or passed as an optional parameter. Will get signal mean data as previous if it has not been obtained.

  eval { $aSignalMeansByPosition = $oSignalMean->signal_means_by_position($position); } or do { croak $EVAL_ERROR; };

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
Carp
npg_qc::api::base
Readonly

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
