#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-06-10
# Last Modified: $Date: 2008-12-01 14:54:22 +0000 (Mon, 01 Dec 2008) $
# Id:            $Id: controller.pm 3652 2008-12-01 14:54:22Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/npg-qc/tags/release-6.0/lib/npg_qc/controller.pm $
#
package npg_qc::controller;
use strict;
use warnings;
use English qw(-no_match_vars);
use Carp qw(carp croak cluck);
use base qw(ClearPress::controller);
use SangerPaths qw(core);
use Website::Utilities::Profiler;
use SangerWeb;
use POSIX qw(strftime);
use CGI;

use npg_qc::model::create_xml;
use npg_qc::model::cumulative_errors_by_cycle;
use npg_qc::model::error_rate_reference_including_blanks;
use npg_qc::model::error_rate_reference_no_blanks;
use npg_qc::model::error_rate_relative_reference_cycle_nucleotide;
use npg_qc::model::error_rate_relative_sequence_base;
use npg_qc::model::errors_by_cycle;
use npg_qc::model::errors_by_cycle_and_nucleotide;
use npg_qc::model::errors_by_nucleotide;
use npg_qc::model::information_content_by_cycle;
use npg_qc::model::log_likelihood;
use npg_qc::model::main;
use npg_qc::model::most_common_blank_pattern;
use npg_qc::model::most_common_word;
use npg_qc::model::signal_mean;
use npg_qc::model::summary;
use npg_qc::model::swift_summary;
use npg_qc::model::run_config;
use npg_qc::model::run_tile;
use npg_qc::model::tile_score;
use npg_qc::model::tile_all;
use npg_qc::model::move_z;
use npg_qc::model::runlog_sax_handler_simple;
use npg_qc::model::run_log;
use npg_qc::model::run_graph;
use npg_qc::model::cache_query;
use npg_qc::model::run;
use npg_qc::model::image_store;

use npg_qc::view::create_xml;
use npg_qc::view::cumulative_errors_by_cycle;
use npg_qc::view::error;
use npg_qc::view::errors_by_cycle;
use npg_qc::view::main;
use npg_qc::view::move_z;
use npg_qc::view::run_config;
use npg_qc::view::run_tile;
use npg_qc::view::signal_mean;
use npg_qc::view::summary;
use npg_qc::view::swift_summary;
use npg_qc::view::tile_score;
use npg_qc::view::move_z;
use npg_qc::view::run_log;
use npg_qc::view::run_graph;
use npg_qc::view::cache_query;
use npg_qc::view::run;

use npg_qc::graph;
use npg_qc::util;

use npg_common::image::graph;
use npg_common::image::heatmap;

our $VERSION = do { my ($r) = q$Revision: 3652 $ =~ /(\d+)/mx; $r; };

sub handler { ## no critic
  my ($self, $util) = @_;

  my $cgi = $util->cgi() || CGI->new();
  $util->cgi($cgi);
  my $sw        = $self->get_sw($util);
  my $pkg       = $util->config->val('application', 'namespace');
  my $appname   = $pkg || $ENV{SCRIPT_NAME};
  my $error_pkg = sprintf q(%s::view::error), $pkg;
  my ($action, $entity, $aspect, $id) = $self->process_request($util);

  if(!$sw->username() &&
     $cgi->param('pipeline')) {
    #########
    # Casually bypass all security for pipeline purposes.
    # Should probably be coupled with a hostname test for il\d+\-win or il\d+proc
    # Or IP-based restrictions
    #
    $util->username('pipeline');

  } else {
    $util->username(lc $sw->username());
  }

  my $session = $sw->session();
  $session->{$appname} ||= {};
  $util->session($sw->session->{$appname});
  $util->cgi($cgi);
  $util->profiler(Website::Utilities::Profiler->new());

  my $viewobject;

  eval {
    my @entities = split /,/mx, $util->config->val('application','views');
    if(!scalar grep { $_ eq $entity } @entities) {
      croak qq(No such view ($entity));
    }

    my $modelclass  = "${pkg}::model::$entity";
    my $viewclass   = "${pkg}::view::$entity";
    my $modelpk     = $modelclass->primary_key() || q();
    my $modelobject = $modelclass->new({
					util => $util,
					$modelpk?($modelpk => $id):(),
				       });
    if(!$modelobject) {
      croak qq(Failed to load $modelobject);
    }

    $viewobject = $viewclass->new({
				   util   => $util,
				   model  => $modelobject,
				   action => $action,
				   aspect => $aspect,
				  });
    if(!$viewobject) {
      croak qq(Failed to load $viewobject);
    }
    1;
  } or do {
    $viewobject = $self->build_error_object($error_pkg, $util, $EVAL_ERROR, $aspect);
  };

  my $decor = $viewobject->decor();

  if(!$viewobject->isa($error_pkg)) {
    $sw->save_session();
  }

  if($decor) {
    $viewobject->output_buffer($sw->header());
  }

  eval {

    $viewobject->output_buffer($viewobject->render());
    1;
  } or do {

    $viewobject = $self->build_error_object($error_pkg, $util, $EVAL_ERROR, $aspect);
    #########
    # reset headers before printing an error
    #
    $viewobject->output_buffer($sw->header());
    $decor    = 1;
    $viewobject->output_buffer($viewobject->render());
  };

  #########
  # re-test decor in case it's changed by render()
  #
  if($viewobject->decor()) {
    #########
    # assume it's safe to re-open the output stream (Eesh!)
    #
    $viewobject->output_finished(0);

    if($sw->is_dev()) {
      $viewobject->output_buffer($util->profiler->report('html'));
    }

    $viewobject->output_buffer($sw->footer());

  } else {
    #########
    # prepend content-type to output buffer
    #
    if(!$viewobject->output_finished()) {
      print qq(X-Generated-By: npg_qc::controller\n) or croak $OS_ERROR;
      print q(Content-type: ). $viewobject->content_type(). "\n\n" or croak $OS_ERROR;
    }
  }

  $viewobject->output_end();

  if ($util->dbh()) {
     $util->dbh->disconnect();
  }

  if($viewobject->isa($error_pkg)) {
    return 1;
  }

  return 0;
}

sub get_sw {
  my ($self, $util) = @_;

  my $jshost        = sprintf '%s://js%s.sanger.ac.uk', (SangerWeb->protocol()||'http'), (SangerWeb->is_dev() || q());
  my $appname       = $util->config->val('application', 'name') || 'Application';

  my $style         = SangerWeb->is_dev()?q(div.banner{background:#e66;padding:5px}):undef;

  my $title         = sprintf q(%s v%s (%s %s database) [ %s ]),
                              uc $appname,
			      $VERSION,
			      uc $appname,
			      uc $util->dbsection(),
			      (strftime '%Y-%m-%dT%H:%M:%S', localtime);

  my $script_name = $ENV{SCRIPT_NAME} || q();
  my $http_host   = $ENV{HTTP_HOST}   || q();

  my $sw          = SangerWeb->new({
				    title      => $title,
				    banner     => $title,
				    jsfile     => [
				    "$jshost/prototype-1.6.0.2.js",
				    "$jshost/scriptaculous/scriptaculous.js",
						   "$jshost/zebra.js",
						   "$jshost/sorttable.js",
						   "$jshost/jsr_class.js",
						   "$jshost/lightbox/js/lightbox.js",
						   $util->config->val('application','javascript'),
						  ],
				    script     => qq(function addEvent(e,t,fn,c){if(e.addEventListener){e.addEventListener(t,fn,c);return true;}else if(e.attachEvent){var r=e.attachEvent("on"+t,fn);return r;}}\nvar SCRIPT_NAME="$script_name";),
				    stylesheet => ["$jshost/lightbox/css/lightbox.css",$util->config->val('application','stylesheet')],
				    style      => $style,
				    atom       => ["http://$http_host$script_name/run_status/;list_atom"],
				    cgi        => $util->cgi(),
				   });

  return $sw;
}

sub build_error_object {
  my ($self, $error_pkg, $util, $eval_error, $aspect) = @_;

  return ($error_pkg->new({
			   util   => $util,
			   errstr => $EVAL_ERROR,
			   aspect => $aspect,
			  }));
}

1;
__END__

=head1 NAME

npg_qc::controller - npg_qc controller

=head1 VERSION

$LastChangedRevision: 3652 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 handler - run the controller

=head2 get_sw - creates a SangerWeb object for the application

=head2 build_error_object - builds an error view object

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

base
strict
warnings
English
Carp
ClearPress::controller
SangerPaths
Website::Utilities::Profiler
SangerWeb
POSIX
CGI
npg_qc::model::create_xml
npg_qc::model::cumulative_errors_by_cycle
npg_qc::model::error_rate_reference_including_blanks
npg_qc::model::error_rate_reference_no_blanks
npg_qc::model::error_rate_relative_reference_cycle_nucleotide
npg_qc::model::error_rate_relative_sequence_base
npg_qc::model::errors_by_cycle
npg_qc::model::errors_by_cycle_and_nucleotide
npg_qc::model::errors_by_nucleotide
npg_qc::model::information_content_by_cycle
npg_qc::model::log_likelihood
npg_qc::model::main
npg_qc::model::most_common_blank_pattern
npg_qc::model::most_common_word
npg_qc::model::signal_mean
npg_qc::model::summary
npg_qc::model::swift_summary
npg_qc::model::run_config
npg_qc::model::run_tile
npg_qc::model::tile_score
npg_qc::model::tile_all
npg_qc::view::create_xml
npg_qc::view::cumulative_errors_by_cycle
npg_qc::view::error
npg_qc::view::errors_by_cycle
npg_qc::view::main
npg_qc::view::run_config
npg_qc::view::run_tile
npg_qc::view::signal_mean
npg_qc::view::summary
npg_qc::view::swift_summary
npg_qc::view::tile_score
npg_qc::util
npg_common::image::graph
npg_common::image::heatmap

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt> & Roger Pettett, E<lt>rmp@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
