#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-06-10
# Last Modified: $Date: 2008-12-01 14:54:22 +0000 (Mon, 01 Dec 2008) $
# Id:            $Id: errors_by_cycle_and_nucleotide.pm 3652 2008-12-01 14:54:22Z ajb $
# Source:        $Source: /repos/cvs/webcore/SHARED_docs/cgi-bin/docrep,v $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/npg-qc/tags/release-6.0/lib/npg_qc/model/errors_by_cycle_and_nucleotide.pm $
#

package npg_qc::model::errors_by_cycle_and_nucleotide;
use strict;
use warnings;
use base qw(npg_qc::model);
use English qw{-no_match_vars};
use Carp;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3652 $ =~ /(\d+)/mx; $r; };

__PACKAGE__->mk_accessors(__PACKAGE__->fields());

sub fields {
  return qw(
            id_errors_by_cycle_and_nucleotide
            id_run_tile
            cycle
            read_as
            really_a
            really_c
            really_g
            really_t
            rescore
          );
}

sub init {
  my $self = shift;
  
  if($self->id_run_tile() &&
     $self->read_as() &&
     $self->cycle() &&
     defined $self->rescore() &&
     !$self->id_errors_by_cycle_and_nucleotide()) {

    my $query = q(SELECT id_errors_by_cycle_and_nucleotide
                 FROM errors_by_cycle_and_nucleotide
                 WHERE id_run_tile = ?
                 AND cycle = ?
                 AND read_as = ?
                 AND rescore = ?
                 );

    my $ref   = [];

    eval {
      $ref = $self->util->dbh->selectall_arrayref($query, {}, $self->id_run_tile(), $self->cycle(), $self->read_as(), $self->rescore());
      1;
    } or do {
      carp $EVAL_ERROR;
      return;
    };

    if(@{$ref}) {
      $self->id_errors_by_cycle_and_nucleotide($ref->[0]->[0]);
    }
  }
  return 1;
}
sub run_tile {
  my $self  = shift;
  my $pkg   = 'npg_qc::model::run_tile';
  return $pkg->new({
		    'util' => $self->util(),
		    'id_run_tile' => $self->id_run_tile(),
		   });
}

1;
__END__
=head1 NAME

npg_qc::model::errors_by_cycle_and_nucleotide

=head1 VERSION

$LastChangedRevision: 3652 $

=head1 SYNOPSIS

  my $oErrorByCycleAndNucleotide = npg_qc::model::errors_by_cycle_and_nucleotide->new({util => $util});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 fields - return array of fields, first of which is the primary key

  my $aFields = $oErrorByCycleAndNucleotide->fields();

=head2 run_tile - returns run_tile object that this object belongs to

  my $oRunTile = $oErrorByCycleAndNucleotide->run_tile();
  
=head2 init

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
npg_qc::model

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
