#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-06-10
# Last Modified: $Date: 2008-12-01 14:54:22 +0000 (Mon, 01 Dec 2008) $
# Id:            $Id: swift_report.pm 3652 2008-12-01 14:54:22Z ajb $
# Source:        $Source: /repos/cvs/webcore/SHARED_docs/cgi-bin/docrep,v $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/npg-qc/tags/release-6.0/lib/npg_qc/model/swift_report.pm $
#

package npg_qc::model::swift_report;
use strict;
use warnings;
use base qw(npg_qc::model);
use npg_qc::model::run_tile;
use Statistics::Lite qw(:all);
use Readonly;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3652 $ =~ /(\d+)/mx; $r; };

Readonly our $PERCENTAGE => 100;

__PACKAGE__->mk_accessors(__PACKAGE__->fields());

sub fields {
  return qw(
            id_swift_report
            id_run_tile
            pf_bases
            pf_good_bases
            pf_reads
            pf_unique
            npf_bases
            npf_good_bases
            npf_reads
            npf_unique
          );
}

sub run_tile {
  my $self  = shift;
  my $pkg   = 'npg_qc::model::run_tile';
  return $pkg->new({
		    'util' => $self->util(),
		    'id_run_tile' => $self->id_run_tile(),
		   });
}

sub swift_reports_by_id_run {
  my ($self, $id_run) = @_;
  my $pkg = ref$self;
  my $query = qq{SELECT @{[join q(, ), map { "sr.$_ AS $_" }  $pkg->fields()]}, rt.tile AS tile, rt.position AS position
                 FROM   @{[$pkg->table]} sr,
                        run_tile rt
                 WHERE  rt.id_run = ?
                 AND    rt.id_run_tile = sr.id_run_tile
                 ORDER BY rt.position, rt.tile};
  return $self->gen_getarray($pkg, $query, $id_run);
}

sub aggregated_swift_reports {
  my ($self, $id_run) = @_;
  if (!$self->{aggregated_swift_reports}) {
    my $lanes = {};
    foreach my $tile (@{$self->swift_reports_by_id_run($id_run)}) {
      push @{$lanes->{$tile->{position}}}, $tile;
    }
    my $return_array = [];
    foreach my $position (sort keys %{$lanes}) {
      my (@pf_bases, @pf_good_bases, @pf_reads, @pf_unique, @npf_bases, @npf_good_bases, @npf_reads, @npf_unique);
      foreach my $tile (@{$lanes->{$position}}) {
        push @pf_bases, $tile->{pf_bases};
        push @pf_good_bases, $tile->{pf_good_bases};
        push @pf_reads, $tile->{pf_reads};
        push @pf_unique, $tile->{pf_unique};
        push @npf_bases, $tile->{npf_bases};
        push @npf_good_bases, $tile->{npf_good_bases};
        push @npf_reads, $tile->{npf_reads};
        push @npf_unique, $tile->{npf_unique};
      }
      my $pf_bases       = sum(@pf_bases);
      my $pf_good_bases  = sum(@pf_good_bases);
      my $pf_error_rate  = sprintf '%.02f', ($PERCENTAGE - ($pf_good_bases*$PERCENTAGE/$pf_bases));
      my $pf_reads       = sum(@pf_reads);
      my $pf_unique      = sum(@pf_unique);
      my $pf_uniq_align  = sprintf '%.02f', ($pf_unique*$PERCENTAGE/$pf_reads);
      my $npf_bases      = sum(@npf_bases);
      my $npf_good_bases = sum(@npf_good_bases);
      my $npf_error_rate = sprintf '%.02f', ($PERCENTAGE - ($npf_good_bases*$PERCENTAGE/$npf_bases));
      my $npf_reads      = sum(@npf_reads);
      my $npf_unique     = sum(@npf_unique);
      my $npf_uniq_align = sprintf '%.02f', ($npf_unique*$PERCENTAGE/$npf_reads);
      push @{$return_array}, [$position, $pf_bases, $pf_good_bases, $pf_error_rate, $pf_reads, $pf_unique, $pf_uniq_align, $npf_bases, $npf_good_bases, $npf_error_rate, $npf_reads, $npf_unique, $npf_uniq_align];
    }
    $self->{aggregated_swift_reports} = $return_array;
  }
  return $self->{aggregated_swift_reports};
}

1;
__END__
=head1 NAME

npg_qc::model::swift_report

=head1 VERSION

$LastChangedRevision: 3652 $

=head1 SYNOPSIS

  my $oSwiftReport = npg_qc::model::swift_report->new({util => $util});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 fields - return array of fields, first of which is the primary key

  my $aFields = $oSwiftReport->fields();

=head2 run_tile - returns run_tile object that this object belongs to

  my $oRunTile = $oSwiftReport->run_tile();

=head2 aggregated_swift_reports - aggregates all the swift report data for individual tiles and returns an array of lanes for viewing in a table

  my $aAggregatedSwiftReports = $oSwiftReport->aggregated_swift_reports($id_run);

=head2 swift_reports_by_id_run - returns all the swift reports for a run, ordered by lane and then tile

  my $aSwiftReportsByIdRun = $oSwiftReport->swift_reports_by_id_run($id_run);

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
npg_qc::model
npg_qc::model::run_tile

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
