#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-06-10
# Last Modified: $Date: 2008-12-01 14:54:22 +0000 (Mon, 01 Dec 2008) $
# Id:            $Id: errors_by_cycle.pm 3652 2008-12-01 14:54:22Z ajb $
# Source:        $Source: /repos/cvs/webcore/SHARED_docs/cgi-bin/docrep,v $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/npg-qc/tags/release-6.0/lib/npg_qc/view/errors_by_cycle.pm $
#

package npg_qc::view::errors_by_cycle;
use strict;
use warnings;
use base qw(npg_qc::view);
use Readonly;
use English qw{-no_match_vars};
use Carp;
use npg_common::image_map;
use npg_common::image::heatmap;
use npg_common::image::graph;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3652 $ =~ /(\d+)/mx; $r; };

Readonly our $WIDTH                => 600;
Readonly our $HEIGHT               => 350;
Readonly our $X_LABEL_SKIP         => 5;
Readonly our $X_LABEL_SKIP_THUMB   => 11;
Readonly our $MAX_ERROR_PERCENTAGE => 20;
Readonly our $THUMB_HEIGHT         => 86;
Readonly our $THUMB_WIDTH          => 126;
Readonly our $THUMBNAIL_TILE_WIDTH => 4;

sub list {
  my ($self) = @_;
  return $self->list_refactor();
}

sub list_heatmaps{
  my ($self) = @_;
  return $self->list_refactor();
}

sub list_ave_by_position {
  my ($self) = @_;
  return $self->list_refactor();
}

sub list_ec_scale_png {
  my ($self) = @_;
  return $self->read_scale_png({ orientation => 'horizontal', end_text => '>20', });
}

sub list_heatmap_with_hover_ajax {
  my ($self) = @_;
  my $util      = $self->util();
  my $model     = $self->model();
  my $cgi       = $util->cgi();
  my $cycle_ref = $cgi->param('cycle_ref');
  my ($id_run, $cycle) = split /_/xms, $cycle_ref;
  $model->{id_run} = $id_run;
  $model->cycle($cycle);

  my $data_refs = {
    id_run    => $id_run,
    end       => 1,
    dataset   => q{error_per_tile},
    image_url => $cgi->param('url'),
    id        => q{error_per_tile:} . $id_run . q{:} . $cycle,
    hover_map => 1,
  };

  my $rt_obj = npg_qc::model::run_tile->new({util => $self->util()});
  my $run_tiles = $rt_obj->run_tiles_per_run_by_lane_end($id_run);


  eval {
    my $end = 1;
    my $cycle_count = $model->cycle_count();

    if ($cycle > $cycle_count) {
      $cycle -= $cycle_count;
      $end = 2;
    }

    $data_refs->{data} = $self->list_heatmap_png($data_refs);
    foreach my $box (@{$data_refs->{data}}) {
      my $data_information = $box->[-1];
      my $params = q{id_run=} . $id_run . q{&position=} . $data_information->{position} . q{&tile=} . $data_information->{tile} . q{&end=} . $end . q{&cycle=} . $cycle;

      my $run_tile = $run_tiles->[$end-1]->[$data_information->{position}-1]->[$data_information->{tile} -1];
      my $id_run_tile = $run_tile->id_run_tile();

      $box->[-1]->{url} = q{/cgi-bin/prodsoft/npg_qc/npg_qc/run_tile/}.$run_tile->id_run_tile().q{" target="blank" onClick="open_tile_viewer('/cgi-bin/prodsoft/npg_qc/npg_qc/run_tile/}. $id_run_tile.q{;read_tile_viewer');};
    }
    my $image_map_object = npg_common::image_map->new();
    $model->{map} = $image_map_object->render_map($data_refs);
  } or do {
    croak 'Unable to render map: ' . $EVAL_ERROR;
  };

  return 1;
}

sub list_heatmap_png {
  my ($self, $data_refs)    = @_;
  my $util      = $self->util();
  my $model     = $self->model();
  my $cgi       = $util->cgi();
  my $cycle_ref = $cgi->param('cycle_ref');
  my ($id_run, $cycle) = split /_/xms, $cycle_ref;
  
  $model->{id_run} = $id_run;
  $model->cycle($cycle);
  
  my $thumbnail = $cgi->param('thumb') || q{};
  
  my $dataset = $model->heatmap_data_per_cycle();

  my $heatmap_obj = npg_common::image::heatmap->new({
    data_array => $dataset,
  });
  
  my $png;
  
  my $arg_refs = $thumbnail eq 'true' ? {
                                         vertical       => 1,
                                         tile_width     => $THUMBNAIL_TILE_WIDTH,
                                         gradient_style => 'percentage',
                                        }
               :                        {
                                         vertical       => 1,
                                         gradient_style => 'percentage',
                                        }
               ;
  eval {

    $png = $heatmap_obj->plot_illumina_map($arg_refs);

    1;

  } or do {

    croak $EVAL_ERROR;

  };

  if ($data_refs->{hover_map}) {
    return $heatmap_obj->image_map_reference();
  }

  return $png;

}

sub list_png {
  my ($self)       = @_;
  my $util         = $self->util();
  my $model        = $self->model();
  my $cgi          = $util->cgi();
  my $tile_ref     = $cgi->param('tile_ref');
  my ($id_run, $position, $tile) = split /_/xms, $tile_ref;
  my $thumbnail    = $cgi->param('thumb');
  my $pre_chastity = $cgi->param('pre_chastity');
  my $plot_type = $cgi->param('plot_type');
  my $graph_obj    = npg_common::image::graph->new({util => $util});
  my $type         = 'area';

  $model->{id_run} = $id_run;
  my $data  = $tile && $plot_type ? $model->data_for_plot_blank_error($id_run, $position, $tile)
            : $tile && $pre_chastity ? $model->data_for_plot_including_pre_chastity($id_run, $position, $tile)
            : $tile                  ? $model->data_for_plot($id_run, $position, $tile)
            :                          $model->average_perc_for_lane($id_run, $position)
            ;

  if(!scalar @{$data}){
    return 0;
  }

  my $title = $tile && $plot_type ? "Blank Base Error rate by cycle for Run $id_run, Position $position, Tile $tile"
            : $tile ? "Error percentage by cycle for Run $id_run, Position $position, Tile $tile."
            :         "Average Error percentage by cycle for Run $id_run, Position $position."
            ;

  my $args = {
    'width'        => $WIDTH,
    'height'       => $HEIGHT,
    'title'        => $title,
    'x_label_skip' => $X_LABEL_SKIP,
    'x_label'      => 'Cycle',
    'y_label'      => 'Percentage',
    'y_min_value'  => 0,
    'y_max_value'  => $MAX_ERROR_PERCENTAGE,
  };

  if (defined $thumbnail && $thumbnail eq 'true') {
    $args->{height}       = $THUMB_HEIGHT;
    $args->{width}        = $THUMB_WIDTH;
    $args->{x_label_skip} = $X_LABEL_SKIP_THUMB;
    $args->{title}        = undef;
    $args->{x_label}      = undef;
    $args->{y_label}      = undef;
    $args->{legend}       = undef;
  }
  if ($pre_chastity) {
    $args->{legend} = ['pre-chastity', 'post-chastity'];
    $type = 'lines';
  }
  
  if($plot_type){
     $args->{dclrs} = [ qw(blue) ];
  }

  return $graph_obj->plotter($data, $args, $type, undef);
}

1;
__END__
=head1 NAME

npg_qc::view::errors_by_cycle

=head1 VERSION

$LastChangedRevision: 3652 $

=head1 SYNOPSIS

  my $o = npg_qc::view::errors_by_cycle->new({util => $util});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 list - handles the list view

=head2 list_ave_by_position - handles the list average by position

=head2 list_heatmaps - handles the list showing heatmaps

=head2 list_refactor - common code to put the id_run onto the model for list views

=head2 list_heatmap_png - handles rendering a png heatmap image for the information presented

=head2 list_png - handles rendering a png image for the information presented

=head2 list_ec_scale_png - handles rendering a png image for the scale of the heat gradients

=head2 list_heatmap_with_hover_ajax - handles rendering a return ajax html snippet which includes a map to hover over the rendered heatmap png

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Carp
Readonly
English
npg_qc::view
npg_common::image_map
npg_common::image
npg_common::image::heatmap
npg_common::image::graph

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
