#!/software/bin/perl -w
#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-11-24
# Last Modified: $Date: 2008-12-01 14:54:22 +0000 (Mon, 01 Dec 2008) $
# Id:            $Id: get_qcal.pl 3652 2008-12-01 14:54:22Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/npg-qc/tags/release-6.0/scripts/get_qcal.pl $
#

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Readonly;
use Statistics::Lite qw(:all);
use Perl6::Slurp;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3652 $ =~ /(\d+)/mx; $r; };

$ENV{'PATH'} = join q(:), qw(/bin /usr/bin /usr/local/bin);

Readonly our $POSITION_OF_QVAL_TWENTY     => 26;
Readonly our $POSITION_OF_QVAL_TWENTYFIVE => 31;
Readonly our $POSITION_OF_QVAL_THIRTY     => 36;
Readonly our $PERCENTAGE_CALCULATOR       => 1000;

my $file_names = `ls`;

my @filenames = split /\n/xms, $file_names;

foreach my $fn (@filenames) {
  if ($fn =~ /fastqcheck/xms) {
    my $temp = $fn;
    $temp =~ s/\.fastqcheck//gxms;
    my @temp=split/_/xms, $temp;
    print qq(id_run="$temp[0]"\nposition="$temp[1]"\nend="$temp[2]");
    my $input = slurp($fn);
    calculate_values($input);
    print "\n";
  }
}

sub calculate_values {
  my ($input) = @_;

  my @lines = split /\n/xm, $input;
  my @temp = split /\s+/xm, $lines[0];
  my $pf_bases = $temp[2];

  foreach my $line (@lines) {
    @temp = split /\s+/xm, $line;
    if ($temp[0] eq 'Total') {
      pop @temp;
      my @working_array = @temp;
      splice @working_array, 0, $POSITION_OF_QVAL_TWENTY;
      my $perc_at_20plus = sum(@working_array) || 0;
      my $calculated_value = sprintf '%.00f', $pf_bases*$perc_at_20plus/$PERCENTAGE_CALCULATOR;
      $calculated_value ||= 0;
      print qq{twenty="$calculated_value"\n};
      @working_array = @temp;
      splice @working_array, 0, $POSITION_OF_QVAL_TWENTYFIVE;
      my $perc_at_25plus = sum(@working_array) || 0;
      $calculated_value = sprintf '%.00f', $pf_bases*$perc_at_25plus/$PERCENTAGE_CALCULATOR;
      $calculated_value ||= 0;
      print qq{twentyfive="$calculated_value"\n};
      @working_array = @temp;
      splice @working_array, 0, $POSITION_OF_QVAL_THIRTY;
      my $perc_at_30plus = sum(@working_array) || 0;
      $calculated_value = sprintf '%.00f', $pf_bases*$perc_at_30plus/$PERCENTAGE_CALCULATOR;
      $calculated_value ||= 0;
      print qq{thirty="$calculated_value"\n};
      last;
    }
  }

}
__END__
=head1 NAME

get_qcal

=head1 VERSION

$LastChangedRevision: 3652 $

=head1 SYNOPSIS

  A simple script to to get the qcal scores from the archive fastqcheck files

=head1 DESCRIPTION

  prompt> ./get_qcal.pl

=head1 SUBROUTINES/METHODS

=head2 calculate_values - calculates the q20,q25 and q30 values from the files and prints out

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Carp
English
Readonly
Statistics::Lite
Perl6::Slurp

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
