#! /software/solexa/bin/perl -wT
#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-11-13
# Last Modified: $Date: 2008-12-01 14:54:22 +0000 (Mon, 01 Dec 2008) $
# Id:            $Id: get_run_tiles.pl 3652 2008-12-01 14:54:22Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/npg-qc/tags/release-6.0/scripts/get_run_tiles.pl $
#

use strict;
use warnings;
use Carp;

use SangerPaths qw(core ensembl tracking trackingnpg badger);
use npg_qc::api::run;
use npg_qc::api::util;
use LWP::UserAgent;

use English qw{-no_match_vars};

our $VERSION = do { my ($r) = q$LastChangedRevision: 3652 $ =~ /(\d+)/mx; $r; };

$ENV{'PATH'} = join q(:), qw(/bin /usr/bin /usr/local/bin);

main();
0;

sub main {

  #########
  # overide base uri for npg_qc
  my $uri  = q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg_qc/npg_qc/};
  my $util = npg_qc::api::util->new({
    useragent => undef,
    base_uri  => undef,
    parser    => undef,
  });

  my$oRun;

  eval {
    $oRun = npg_qc::api::run->new({util => $util});
    my ($id_run) = $ARGV[0] =~ /(\d+)/xms;
    $oRun->id_run($id_run);
    $oRun->read_run_tiles();
    1;
  } or do {
    croak $EVAL_ERROR;
  };

  my $read_one_lanes  = $oRun->get_read_one_lanes();
  my $read_two_lanes = $oRun->get_read_two_lanes();

  print 'id_run : ' . $oRun->id_run() . "\n";
  print 'is_paired : ' . $oRun->get_is_paired() . "\n";
  print 'id_run_pair : ' . $oRun->get_id_run_pair() . "\n";

  my $lane = $read_one_lanes->[7];
  print "looking at a read_one lane\n";
  print "\tid_run : " . $lane->id_run() . "\n";
  print "\tlane : " . $lane->get_lane() . "\n";
  print "\tread_end : " . $lane->get_read_end() . "\n";

  my $tiles = $lane->get_tiles();
  my $tile = $tiles->[30];
  print "looking at a tile\n";
  print "\t\tid_run : " . $tile->id_run() . "\n";
  print "\t\tlane : " . $tile->get_lane() . "\n";
  print "\t\tread_end : " . $tile->get_read_end() . "\n";
  print "\t\tid_run_tile : " . $tile->get_id_run_tile() . "\n";
  print "\t\ttile : " . $tile->get_tile() . "\n";
  print "\t\trow : " . $tile->get_row() . "\n";
  print "\t\tcol : " . $tile->get_col() . "\n";
  print "\t\tavg_newz : " . $tile->get_avg_newz() . "\n";

  my $tqcd = $tile->get_tile_qc_data();
  print "\ttile_qc_data\n";
  foreach my $k (sort keys %{$tqcd}) {
    $tqcd->{$k} = $tqcd->{$k} || q{};
    print "\t\t$k : $tqcd->{$k}\n";
  }

  $lane = $read_two_lanes->[3];
  print "looking at a read_two lane\n";
  print "\tid_run : " . $lane->id_run() . "\n";
  print "\tlane : " . $lane->get_lane() . "\n";
  print "\tread_end : " . $lane->get_read_end() . "\n";

  $tiles = $lane->get_tiles();
  $tile = $tiles->[67];
  print "looking at a tile\n";
  print "\t\tid_run : " . $tile->id_run() . "\n";
  print "\t\tlane : " . $tile->get_lane() . "\n";
  print "\t\tread_end : " . $tile->get_read_end() . "\n";
  print "\t\tid_run_tile : " . $tile->get_id_run_tile() . "\n";
  print "\t\ttile : " . $tile->get_tile() . "\n";
  print "\t\trow : " . $tile->get_row() . "\n";
  print "\t\tcol : " . $tile->get_col() . "\n";
  print "\t\tavg_newz : " . $tile->get_avg_newz() . "\n";

  $tqcd = $tile->get_tile_qc_data();
  print "\ttile_qc_data\n";
  foreach my $k (sort keys %{$tqcd}) {
    $tqcd->{$k} = $tqcd->{$k} || q{};
    print "\t\t$k : $tqcd->{$k}\n";
  }


}

__END__
=head1 NAME

get_summary

=head1 VERSION

$LastChangedRevision: 3652 $

=head1 SYNOPSIS

  A simple script to use npg_qc::api::run to obtain the run tiles for a run

=head1 DESCRIPTION

  prompt> ./get_run_tiles.pl <id_run>

  spits out the chip summary information and the lane summary info from read one of the run requested, read two is available.

=head1 SUBROUTINES/METHODS

=head2 main - is called in the script and does everything

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
npg_qc::api:util
npg_qc::api:run
Carp
SangerPaths
LWP::UserAgent
English

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
