#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-06-12
# Last Modified: $Date: 2008-12-01 14:54:22 +0000 (Mon, 01 Dec 2008) $
# Id:            $Id: 30-api-cache_query.t 3652 2008-12-01 14:54:22Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/npg-qc/tags/release-6.0/t/30-api-cache_query.t $
#

use strict;
use warnings;
use Test::More tests => 12;
use English qw(-no_match_vars);
use IO::Scalar;
use lib qw{t};
use SangerPaths qw(ensembl core badger trackingnpg tracking);
use t::api::util;
use t::useragent;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3652 $ =~ /(\d+)/mx; $r; };

use_ok('npg_qc::api::cache_creator');

##
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $util = t::api::util->new({dbh => 'dummy', fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});
  my $cache_creator = npg_qc::api::cache_creator->new({util => $util});

  isa_ok($cache_creator, 'npg_qc::api::cache_creator', '$cache_creator');

  my $response;
  eval { $response = $cache_creator->send_request_to_run_cache_query(); };
  is($EVAL_ERROR, q{}, 'no croak on firing creation request');
  ok($response, 'response is true');
  is($cache_creator->get_response_statement(), 'success', 'success statement stored in object');

  eval { $response = $cache_creator->send_request_to_run_cache_query(1234); };
  is($EVAL_ERROR, q{}, 'no croak on firing creation request wih an id_run');
  ok($response, 'response is true');
  is($cache_creator->get_response_statement(), 'success', 'success statement stored in object');

  $ua_mock->{'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query'} = q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query_not_success.xml};
  
  eval { $response = $cache_creator->send_request_to_run_cache_query(); };
  like($EVAL_ERROR, qr{Response\ not\ good}, 'croaked on firing creation request');
  is($cache_creator->get_response_statement(), 'failure', 'failure statement stored in object');
}

##
{
  my $ua_mock = {};
  my $ua   = t::useragent->new({is_success => 0, mock => $ua_mock});
  my $util = t::api::util->new({dbh => 'dummy', fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});
  my $cache_creator = npg_qc::api::cache_creator->new({util => $util});

  my $response;
  eval { $response = $cache_creator->send_request_to_run_cache_query(); };
  like($EVAL_ERROR, qr{failed\ to\ successfully\ post\ -\ this\ could\ be\ from\ a\ timeout\ issue}, 'croaked on firing creation request');
  is($cache_creator->get_response_statement(), 'no success', q{'no success' statement stored in object});

}
1;
