########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-11-21
# Last Modified: $Date: 2008-12-01 14:54:22 +0000 (Mon, 01 Dec 2008) $
# Id:            $Id: 30-api-image_loader.t 3652 2008-12-01 14:54:22Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/npg-qc/tags/release-6.0/t/30-api-image_loader.t $
#

use strict;
use warnings;
use Test::More tests => 18;
use English qw(-no_match_vars);
use lib qw{t};
use SangerPaths qw(core ensembl);
use t::api::util;
use npg::api::util;
use t::useragent;
use t::util;
use DBI; # ARGH!
use Carp;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3652 $ =~ /(\d+)/mx; $r; };

use_ok('npg_qc::api::image::loader');

my $set_of_image_names = [qw(
    1672_1_all.png  1672_1_call.png  1672_1_base.png  1672_2_all.png  1672_2_call.png  1672_2_base.png
    1672_3_all.png  1672_3_call.png  1672_3_base.png  1672_4_all.png  1672_4_call.png  1672_4_base.png
    1672_5_all.png  1672_5_call.png  1672_5_base.png  1672_6_all.png  1672_6_call.png  1672_6_base.png
    1672_7_all.png  1672_7_call.png  1672_7_base.png  1672_8_all.png  1672_8_call.png  1672_8_base.png
    )];
my $ivc_image_dir_root = q{t/data/ivc_images};
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg/npg/}   => q{},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => undef, npg_api_util => $npg_api_util, fixtures => 1, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg/npg/'});
  $util->load_fixtures();
  my $test_dbi = $util->get_dbh();
  isa_ok($test_dbi, q{DBI::db}, 'util has test db');
  my $loader = npg_qc::api::image::loader->new({util => $util});

  is($loader->util->get_dbh(), $test_dbi, 'loader has test db through test util');
  isa_ok($loader, 'npg_qc::api::image::loader', '$loader');
  $loader->set_dir_root($ivc_image_dir_root);
  $loader->set_image_names($set_of_image_names);
  is($loader->get_dir_root(), 't/data/ivc_images', 'dir_root stored ok');
  my $image_names = $loader->get_image_names();
  isa_ok($image_names, 'ARRAY', '$loader->get_images_names()');
  is(scalar@{$image_names}, 24, 'array length is correct');

  $loader->set_thumbnail(1);
  $loader->set_id_run(1672);
  $loader->set_type('signal_mean');

  eval { $loader->image_loop_from_file(); };
  is($EVAL_ERROR, q{}, 'no croak on $loader->image_loop');
  eval { $loader->load_into_database(); };
  like($EVAL_ERROR, qr{DBD::mysql::db\ do\ failed:\ Duplicate\ entry\ '1672-signal_mean-1-1672_8_base.png'\ for\ key}, 'expecting to croak due to duplicate entry');

}
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg/npg/}   => q{},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => undef, npg_api_util => $npg_api_util, fixtures => 1, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg/npg/'});
  $util->load_fixtures();
  my $test_dbi = $util->get_dbh();
  isa_ok($test_dbi, q{DBI::db}, 'util has test db');
  my $loader = npg_qc::api::image::loader->new({util => $util});

  is($loader->util->get_dbh(), $test_dbi, 'loader has test db through test util');
  $loader->set_dir_root($ivc_image_dir_root);
  $loader->set_image_names($set_of_image_names);
  $loader->set_thumbnail(0);
  $loader->set_id_run(1672);
  $loader->set_suffix('gif');
  $loader->set_type('signal_mean');

  eval { $loader->image_loop_from_file(); };
  is($EVAL_ERROR, q{}, 'no croak on $loader->image_loop');
  eval { $loader->load_into_database(); };
  like($EVAL_ERROR, qr{DBD::mysql::db\ do\ failed:\ Duplicate\ entry\ '1672-signal_mean-0-1672_8_base.png'\ for\ key}, 'expecting to croak due to duplicate entry');
  $loader->set_id_run(1670);
  eval { $loader->load_into_database(); };
  is($EVAL_ERROR, q{}, 'no croak when id_run is different');
  $loader->set_type('test');
  eval { $loader->load_into_database(); };
  is($EVAL_ERROR, q{}, 'no croak when type is different');
  $loader->set_thumbnail(1);
  eval { $loader->load_into_database(); };
  is($EVAL_ERROR, q{}, 'no croak when thumbnail is different');
  $loader->set_current_image_name('test');
  $loader->set_suffix(undef);
  eval { $loader->load_into_database(); };
  is($EVAL_ERROR, q{}, 'no croak when image_name is different and suffix not provided by image name or manual set');
  eval {$loader->image_loop_from_file(); };
  like($EVAL_ERROR, qr{DBD::mysql::db\ do\ failed:\ Duplicate\ entry\ '1670-test-1-1672_8_base.png'\ for\ key}, 'expecting croak due to duplicate loading');
}
