#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-09-29
# Last Modified: $Date: 2008-12-01 14:54:22 +0000 (Mon, 01 Dec 2008) $
# Id:            $Id: 30-api-signal_mean.t 3652 2008-12-01 14:54:22Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/npg-qc/tags/release-6.0/t/30-api-signal_mean.t $
#

use strict;
use warnings;
use Test::More tests => 32;
use English qw(-no_match_vars);
use IO::Scalar;
use lib qw{t};
use SangerPaths qw(core ensembl);
use t::api::util;
use npg::api::util;
use t::useragent;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3652 $ =~ /(\d+)/mx; $r; };

use_ok('npg_qc::api::signal_mean');

{
  my $ua_mock = {};

  my $ua   = t::useragent->new({is_success => 0, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg/npg/'});
  my $signal_mean = npg_qc::api::signal_mean->new({util => $util});
  isa_ok($signal_mean, 'npg_qc::api::signal_mean', '$signal_mean');
  my $signal_means;
  my @fields = $signal_mean->fields();
  is(scalar@fields, 15, 'fields list contains 16 fields');
  $signal_mean->id_run(1720);
  eval { $signal_mean->read_signal_mean_data();  };
  like($EVAL_ERROR, qr{error\ in\ t::useragent}, 'croaked as error in useragent response');
}

{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//signal_mean.json?id_run=1720}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/signal_mean_id_run_1720.json},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg/npg/'});
  my $signal_mean = npg_qc::api::signal_mean->new({util => $util});
  $signal_mean->id_run(1720);
  eval { $signal_mean->read_signal_mean_data();  };
  is($EVAL_ERROR, q{}, 'no croak for $signal_mean->read_signal_mean_data()');

  my $signal_means = $signal_mean->get_signal_means();
  isa_ok($signal_means, 'ARRAY', '$signal_mean->get_signal_means()');
  is(scalar@{$signal_means}, 591, 'correct number of signal means obtained');

  my $fsm = $signal_means->[0];
  isa_ok($fsm, 'npg_qc::api::signal_mean');
	is($fsm->get_id_run(), 1720, '$fsm->get_id_run() is correct');
	is($fsm->get_position(), 1, '$fsm->get_position() is correct');
	is($fsm->get_all_a(), '335.70', '$fsm->get_all_a() is correct');
	is($fsm->get_all_c(), '98.60', '$fsm->get_all_c() is correct');
	is($fsm->get_all_g(), '227.40', '$fsm->get_all_g() is correct');
	is($fsm->get_all_t(), '273.90', '$fsm->get_all_t() is correct');
	is($fsm->get_base_a(), '37.60', '$fsm->get_base_a() is correct');
	is($fsm->get_base_c(), '10.70', '$fsm->get_base_c() is correct');
	is($fsm->get_base_g(), '25.50', '$fsm->get_base_g() is correct');
	is($fsm->get_base_t(), '26.30', '$fsm->get_base_t() is correct');
	is($fsm->get_call_a(), '853.20', '$fsm->get_call_a() is correct');
	is($fsm->get_call_c(), '703.60', '$fsm->get_call_c() is correct');
	is($fsm->get_call_g(), '882.20', '$fsm->get_call_g() is correct');
	is($fsm->get_call_t(), '854.10', '$fsm->get_call_t() is correct');
	is($fsm->get_cycle(), 1, '$fsm->get_cycle() is correct');

  my $position_3_sms = $signal_mean->signal_means_by_position(3);
  isa_ok($position_3_sms, 'ARRAY', '$signal_mean->signal_means_by_position(3)');
  is(scalar@{$position_3_sms}, 74, 'correct number of cycles');
  isa_ok($position_3_sms->[5], 'npg_qc::api::signal_mean', '$signal_mean->signal_means_by_position(3)->[5]');
  is($position_3_sms->[5]->get_position(), 3, 'position is 3');
}
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//signal_mean.json?id_run=1720}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/signal_mean_id_run_1720.json},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg/npg/'});
  my $signal_mean = npg_qc::api::signal_mean->new({util => $util});

  $signal_mean->id_run(1720);
  $signal_mean->set_position(7);

  my $position_7_sms;
  eval { $position_7_sms = $signal_mean->signal_means_by_position(); };
  is($EVAL_ERROR, q{}, 'no croak running $signal_mean->signal_means_by_position() when no read_signal_mean_data done first and position has been set into the object - value 7');

  isa_ok($position_7_sms, 'ARRAY', '$signal_mean->signal_means_by_position()');
  is(scalar@{$position_7_sms}, 74, 'correct number of cycles');
  isa_ok($position_7_sms->[20], 'npg_qc::api::signal_mean', '$position_7_sms->[20]');
  is($position_7_sms->[20]->get_position(), 7, 'position is 7');
}
